@@ -49,3 +49,19 @@ test_that("S3 summary method with filled dag", {
 
   expect_snapshot_output(summary(dag))
 })
+
+test_that("S3 summary method with dag filled with formulas", {
+
+  dag <- empty_dag() +
+    node("A", type="rbernoulli", p=0.1) +
+    node("B", type="rbernoulli", p=0.2) +
+    node("C", type="gaussian", formula= ~ -10 + A*0.1 + B*0.2, error=10) +
+    node("D", type="binomial", formula= ~ -5 + B*7 + C*1) +
+    node("E", type="poisson", formula= ~ -2 + A*4 + C*3 + A:B*2) +
+    node("F", type="negative_binomial", formula= ~ -2 + A*4 + C*3 + A:B*2,
+         theta=0.34) +
+    node("G", type="cox", formula= ~ A*4 + C*3 + A:B*2,
+         surv_dist="weibull", lambda=10, gamma=0.23, cens_dist=NULL)
+
+  expect_snapshot_output(summary(dag))
+})
