@@ -15,6 +15,23 @@ test_that("wrong use_saved_states", {
   expect_error(sim2data(sim, to="long", use_saved_states=1))
 })
 
+test_that("wrong overlap", {
+  expect_error(sim2data(sim, to="long", overlap="left"),
+               "'overlap' must be either TRUE or FALSE.")
+})
+
+test_that("wrong keep_only_first", {
+  expect_error(sim2data(sim, to="long", keep_only_first=NULL),
+               "'keep_only_first' must be either TRUE or FALSE.")
+})
+
+test_that("wrong target_event", {
+  expect_error(sim2data(sim, to="long", target_event="sickness"),
+               paste0("'target_event' must be a single character ",
+                      "string, specifying a time_to_event node used ",
+                      "in the creation of 'sim'."))
+})
+
 test_that("use_saved_states='all' when not done", {
   expect_error(sim2data(sim, to="long", use_saved_states=TRUE))
 })
